unit uJCompression;

interface

uses SysUtils, Classes, contnrs;

type

  { this is what type of support the compression object has
    cmNone = No zip support available (most likely required DLLs were not found
    cmRead = Read only/Extract support is only available
    cmWrite = Write/Create File support is only available
    cmFull = Full support of extraction and creation of files is available
  }
  TJCompressMode = (cmNone, cmRead, cmWrite, cmFull);

  EJCompressException = class(Exception); //generic compression exception

  IJCompressible = interface
    ['{08F3CA94-6C39-41E0-841D-85F376AEE959}']
    Function  GetCurrentMode(): TJCompressMode;
    Function  SupportsFileName(Input : TFileName) : Boolean; //based on full filename
    Procedure GetContentsOfFile(var Contents : TStrings; FileName : TFileName);
    procedure RenameContentToFile(FileName : TFileName);
    procedure RenameFileToContent(FileName : TFileName);
    procedure UnCompressSingleFile(input : TFileName; OutDir : String; ExtrFile : TFileName);
    Function  UnCompressROM(var ZipContents : TStringList; Zip : TFileName) : String;
    Procedure UnCompressFile(input : TFileName; OutDir : String; Opt_Diff, Opt_Path, Opt_OW : boolean);
    Function  VerifyFile(Z_File : TFileName) : Boolean;
    procedure CompressDirectory(Directory : String; FileName : TFileName; Filter : TStrings);
    procedure CompressSingleFile(NewFile : TFileName; OutDir : String; CompLevel : Integer = 9);
    Property CurrentMode : TJCompressMode read GetCurrentMode;
  end;

  TJCompression = class(TInterfacedObject, IJCompressible)
  private
    _compressors : TObjectList;
    Function IndexOfCompressor(FileName : String) : Integer;
    Function GetCurrentMode() : TJCompressMode;
  public
    constructor Create();
    destructor destroy(); override;
    //interface functions
    procedure CompressDirectory(Directory : String; FileName : TFileName; Filter : TStrings);
    procedure CompressSingleFile(NewFile : TFileName; OutDir : String; CompLevel : Integer = 9);
    Procedure GetContentsOfFile(var Contents : TStrings; FileName : TFileName);
    procedure RenameContentToFile(FileName : TFileName);
    procedure RenameFileToContent(FileName : TFileName);
    Function  SupportsFileName(Input : TFileName) : Boolean; //based on full filename
    procedure UnCompressSingleFile(input : TFileName; OutDir : String; ExtrFile : TFileName);
    Function  UnCompressROM(var ZipContents : TStringList; Zip : TFileName) : String;
    Procedure UnCompressFile(input : TFileName; OutDir : String; Opt_Diff, Opt_Path, Opt_OW : boolean);
    Function  VerifyFile(Z_File : TFileName) : Boolean;
  end;

implementation

uses uJCompressZip;

Constructor TJCompression.Create();
begin
  inherited;
  _Compressors := TObjectList.Create(True);
  _Compressors.Add(TJZip.Create);

end;

destructor TJCompression.destroy();
begin
  _Compressors.Clear;
  _Compressors.Free;
  inherited;
end;

// Private Routines

Function TJCompression.IndexOfCompressor(FileName : String) : Integer;
var
  I : Integer;
  MyCompress : IJCompressible;
begin
  Result := -1;
  for i := 0 to _compressors.Count-1 do
  begin
    _Compressors[i].GetInterface(IJCompressible, MyCompress);
    if (MyCompress <> nil) and (MyCompress.SupportsFileName(FileName)) then
      Result := i
    else
      continue;
  end;
end;

Function TJCompression.GetCurrentMode() : TJCompressMode;
begin
  Result := cmNone;
end;

Function  TJCompression.SupportsFileName(Input : TFileName) : Boolean; //based on full filename
begin
  if Self.IndexOfCompressor(Input) <> -1 then
    Result := True
  else
    Result := False;
end;

Procedure TJCompression.GetContentsOfFile(var Contents : TStrings; FileName : TFileName);
var
  I : Integer;
  MyCompress : IJCompressible;
begin
  I := Self.IndexOfCompressor(FileName);
  if i <> -1 then
  begin
    _Compressors[i].GetInterface(IJCompressible, MyCompress);
    if MyCompress.CurrentMode in [cmRead, cmFull] then
      MyCompress.GetContentsOfFile(Contents, FileName);
  end
  else
    raise EJCompressException.Create('Compressor not found for this file');
end;

procedure TJCompression.RenameContentToFile(FileName : TFileName);
var
  I : Integer;
  MyCompress : IJCompressible;
begin
  I := Self.IndexOfCompressor(FileName);
  if i <> -1 then
  begin
    _Compressors[i].GetInterface(IJCompressible, MyCompress);
    if MyCompress.CurrentMode in [cmWrite, cmFull] then
      MyCompress.RenameContentToFile(FileName);
  end
  else
    raise EJCompressException.Create('Compressor not found for this file');
end;

procedure TJCompression.RenameFileToContent(FileName : TFileName);
var
  I : Integer;
  MyCompress : IJCompressible;
begin
  I := Self.IndexOfCompressor(FileName);
  if i <> -1 then
  begin
    _Compressors[i].GetInterface(IJCompressible, MyCompress);
    if MyCompress.CurrentMode in [cmRead, cmFull] then
      MyCompress.RenameFileToContent(FileName);
  end
  else
    raise EJCompressException.Create('Compressor not found for this file');
end;

procedure TJCompression.UnCompressSingleFile(input : TFileName; OutDir : String; ExtrFile : TFileName);
var
  I : Integer;
  MyCompress : IJCompressible;
begin
  I := Self.IndexOfCompressor(input);
  if i <> -1 then
  begin
    _Compressors[i].GetInterface(IJCompressible, MyCompress);
    if MyCompress.CurrentMode in [cmRead, cmFull] then
      MyCompress.UnCompressSingleFile(input, outdir, extrfile);
  end
  else
    raise EJCompressException.Create('Compressor not found for this file');
end;

Function  TJCompression.UnCompressROM(var ZipContents : TStringList; Zip : TFileName) : String;
var
  I : Integer;
  MyCompress : IJCompressible;
begin
  I := Self.IndexOfCompressor(Zip);
  if i <> -1 then
  begin
    _Compressors[i].GetInterface(IJCompressible, MyCompress);
    if MyCompress.CurrentMode in [cmRead, cmFull] then
      MyCompress.UnCompressROM(ZipContents, Zip);
  end
  else
    raise EJCompressException.Create('Compressor not found for this file');
end;

Procedure TJCompression.UnCompressFile(input : TFileName; OutDir : String; Opt_Diff, Opt_Path, Opt_OW : boolean);
var
  I : Integer;
  MyCompress : IJCompressible;
begin
  I := Self.IndexOfCompressor(input);
  if i <> -1 then
  begin
    _Compressors[i].GetInterface(IJCompressible, MyCompress);
    if MyCompress.CurrentMode in [cmRead, cmFull] then
      MyCompress.UnCompressFile(input, outdir, opt_diff, opt_path, opt_ow);
  end
  else
    raise EJCompressException.Create('Compressor not found for this file');
end;

Function  TJCompression.VerifyFile(Z_File : TFileName) : Boolean;
var
  I : Integer;
  MyCompress : IJCompressible;
begin
  Result := True;
  I := Self.IndexOfCompressor(Z_File);
  if i <> -1 then
  begin
    _Compressors[i].GetInterface(IJCompressible, MyCompress);
    if MyCompress.CurrentMode in [cmRead, cmFull] then
      Result := MyCompress.VerifyFile(Z_File);
  end
  else
    raise EJCompressException.Create('Compressor not found for this file');
end;

procedure TJCompression.CompressDirectory(Directory : String; FileName : TFileName; Filter : TStrings);
var
  I : Integer;
  MyCompress : IJCompressible;
begin
  I := Self.IndexOfCompressor(FileName);
  if i <> -1 then
  begin
    _Compressors[i].GetInterface(IJCompressible, MyCompress);
    if MyCompress.CurrentMode in [cmWrite, cmFull] then
      MyCompress.CompressDirectory(Directory, FileName, Filter);
  end
  else
    raise EJCompressException.Create('Compressor not found for this file');
end;

procedure TJCompression.CompressSingleFile(NewFile : TFileName; OutDir : String; CompLevel : Integer = 9);
var
  I : Integer;
  MyCompress : IJCompressible;
begin
  I := Self.IndexOfCompressor(NewFile);
  if i <> -1 then
  begin
    _Compressors[i].GetInterface(IJCompressible, MyCompress);
    if MyCompress.CurrentMode in [cmWrite, cmFull] then
      MyCompress.CompressSingleFile(NewFile, OutDir, CompLevel);
  end
  else
    raise EJCompressException.Create('Compressor not found for this file');
end;

end.
