unit uRom;

interface

uses SysUtils, IniFiles, Classes, Contnrs;

type

  Tips = record
    Index : Integer;
    Name : string;
    Path : String;
  end;

  TQPExportOpt = Record
    AddName : Boolean;
    AddPath : Boolean;
    AddEmu : Boolean;
    AddWWW : Boolean;
    AddYear : Boolean;
    AddType : Boolean;
    AddLanguage : Boolean;
    AddRating : Boolean;
    AddPlayers : Boolean;
  end;

  TQPRom = class
  protected
    _name, //this is the name used for display purposes
    _CloneName, //Used Internally mainly for managing MAME clones.
    _ParentName, //Used Internally for storing the Parent of a Clone.
    _ZipName, //Used Internally to store which file inside a zip file is the ROM
    _path, //the path to the rom.
    _emulator, //the Emulator this rom is linked to
    _Company, //The company who made the game.
    _Year,     //the year this ROM first came out
    _GameType, //The type of game
    _MultiPlayer,  //The type of Multiplayer game.
    _Rating,   //A user rating.
    _Language, //The language of this ROM
    _Comment, //Any miscellaneous comments you might want to store.
    _Parameters : String; //Any additional paramters to be used when running
    _NumPlay : integer;   //The amount of times this rom has been played
    _Overwrite : Boolean; //Option to overwrite any emulator specific params with ROM params

    _IconIndex : Integer; //Used Internally to manage the ROMs iconindex - for MAME roms mostly.
    _Exists : Integer; //Used Internally to speed up VTRom Image finding. 1 - Exists 0 - No -1 - Not set.
    
    Function GetFileNameNoExt() : String;
    Function GetSummary() : String;

  public
    IPS : array of Tips;
    Constructor Create(asString : String);overload;
    property Name:String read _Name write _Name;
    Property CloneName : String read _CloneName write _CloneName;
    Property ParentName : String Read _ParentName write _ParentName;
    Property ZipName : String Read _ZipName write _ZipName;
    property Path:String read _Path write _path;
    property Emulator:String read _emulator write _emulator;
    Property Company : String read _Company write _Company;
    property Year:String read _Year write _Year;
    property GameType:String read _GameType write _GameType;
    Property Multiplayer : String Read _Multiplayer write _Multiplayer;
    property Rating:String read _Rating write _Rating;
    property Language:String read _Language write _Language;
    property Parameters:String read _Parameters write _Parameters;
    Property Comment : String read _Comment write _Comment;
    property NumPlay:Integer read _NumPlay write _NumPlay;
    property Overwrite:Boolean read _Overwrite write _Overwrite;

    property IconIndex:Integer read _IconIndex write _IconIndex;
    Property Exists : Integer read _Exists write _Exists;

    Property FileNameNoExt : String Read GetFileNameNoExt;

    //This returns a nicely formatted description of the ROMs properties.  for use in a hint *)
    Property Summary : String Read GetSummary;

    procedure Assign(Source : TQPRom);
    procedure BrowseToROM();
    procedure FromString(strRom : String);
    Function MatchesQuery(Query : TObjectList; MatchAll : Boolean) : Boolean;
    procedure RenameFile(NewName : String);
    procedure RunIPS(ExtrDir : String; Emu : TObject; IPSIndex : Integer);
    Procedure SetLanguage();
    Procedure SetMameGameCategory(Ini : TMemIniFile);
    procedure SetMameMultiplayer(Ini : TMemIniFile);
    Procedure ToCSV(var CodeList : TStrings; Opt : TQPExportOpt);
    Procedure ToHTML(var CodeList : TStrings; EList : TObjectList; Opt : TQPExportOpt);
    Function ToString() : String;
    Procedure ToXML(var CodeList : TStrings; EList : TObjectList; Opt : TQPExportOpt);

  end;

implementation

{$I Compilers.inc}

uses JCLShell, JCLStrings, uIPSpatch, uJZip, uJUtilities,
      uJFile, uEmu, FindFile, uEmuList, uQuery, uQPConst;

{ TQPRom }

Constructor TQPRom.Create(asString : String);
begin
  Self.FromString(asString);
end;

procedure TQPRom.Assign(Source : TQPRom);
var
  i : Integer;
begin
  _name := Source.Name;
  _CloneName := Source.CloneName;
  _ParentName := Source.ParentName;
  _ZipName := Source.ZipName;
  _path := Source.Path;
  _emulator := Source.Emulator;
  _Year := Source.Year;
  _GameType := Source.GameType;
  _MultiPlayer := Source.Multiplayer;
  _Rating := Source.Rating;
  _Language := Source.Language;
  _Parameters := Source.Parameters;
  _NumPlay := Source.NumPlay;
  _Overwrite := Source.Overwrite;
  _IconIndex := Source.IconIndex;
  _Exists := Source.Exists;
  _Company := Source.Company;
  _Comment := Source.Comment;
  //IPS := Copy(Source.IPS);  for some reason this line caused a L681 internal error
  SetLength(Self.IPS, Length(Source.IPS));
  for i := 0 to Length(Source.IPS)-1 do
  begin
    Self.IPS[i].Index := Source.IPS[i].Index;
    Self.IPS[i].Name := Source.IPS[i].Name;
    Self.IPS[i].Path := Source.IPS[i].Path;
  end;    
end;

{-----------------------------------------------------------------------------}

procedure TQPROM.BrowseToROM();
begin
  if DirectoryExists(ExtractFilePath(_Path)) then
    JCLShell.ShellExecEx(ExtractFilePath(_path))
  else
    raise EJException.create(QP_DIRECTORY_NOT_EXIST);
end;

{-----------------------------------------------------------------------------}

Function TQPROM.GetFileNameNoExt() : String;
begin
  if Self._ZipName <> '' then
    //this rom is inside a zip file and its real name is hidden, we need to use this.
    Result := ExtractFileName(ChangeFileExt(_ZipName, ''))
  else if Self._CloneName <> '' then
      //this rom was scanned using MAME, just return its 'clonename'
      Result := Self._CloneName
  else
    //just return whatever is in the path field.
    Result := ExtractFileName(ChangeFileExt(self._path, ''));
end;

{-----------------------------------------------------------------------------}

Function TQPROM.GetSummary() : String;
begin

  Result := QP_ROM_NAME + ' : ' + _name + #13 +
            QP_ROM_PATH + ' : ' + _path + #13 +
            QP_ROM_EMU + ' : ' + _emulator;

  If _Comment <> '' then
    Result := Result + #13 + QP_ROM_COMMENT + ' : ' + #13 + _Comment;

end;

procedure TQPROM.FromString(strRom : String);
var
  SplitList : TStringList;
  IPSpos, Isize : integer;
  Comment : String;
begin

  //set our internal data
  _IconIndex := -1;
  _Exists := -1;

  //we need to create our splitlist stringlist
  SplitList := TStringList.Create;
  try
    // this function is from jedi code library.
    StrTokenToStrings(strRom, '', SplitList);
    if SplitList.Count > 6 then
    begin
      //exception handler which catches any attempts to read data that doesnt exist in the string.
      try
        _name := SplitList[0];
        _CloneName := SplitList[1];
        _ParentName := SplitList[2];
        _ZipName := SplitList[3];
        //now process the rest of the rom information
        _path := SplitList[4];
        _emulator := SplitList[5];
        _Company := SplitList[6];
        _year := SplitList[7];
        _GameType := SplitList[8];
        _Rating := SplitList[9];
        _Language := SplitList[10];
        _Parameters := SplitList[11];

        //the comment needs to be processed to add in any '#13#10'.
        Comment := SplitList[12];
        StrReplace(Comment, '#13', chr(13), [rfReplaceAll]);
        StrReplace(Comment, '#10', chr(10), [rfReplaceAll]);
        _Comment := Comment;

        _NumPlay := StrToInt(SplitList[13]);
        _Overwrite := StrToBool(SplitList[14]);
        IPSpos := 16;
        Isize := 0;
        while SplitList[IPSpos] <> '</IPS>' do
        begin
          inc(ISize);
          SetLength(Ips, ISize);
          IPS[ISize-1].Index := Isize-1;
          IPS[Isize-1].Name := SplitList[IPSpos];
          inc(IPSpos);
          self.IPS[Isize-1].Path := SplitList[IPSpos];
          inc(IPSpos);
        end;

        _Multiplayer := SplitList[IPSpos+1];

      except
        Exit; //ignore any errors we get in here.. 
      end;

    end;

  finally
    FreeAndNil(SplitList);
  end;

end;

{-----------------------------------------------------------------------------}

Function TQPRom.MatchesQuery(Query : TObjectList; MatchAll : Boolean) : Boolean;
var
  I : Integer;
  iQuery : TQPQuery;

      Function RomCmp(searchStr, MainStr : String; FullMatch : Boolean) : Boolean;
      begin
        Result := False;
        if FullMatch then
        begin
          if StrCompare(searchStr, MainStr) = 0 then
            Result := True;
        end
        else
        begin
          if StrIPos(searchStr, MainStr) <> 0 then
            Result := True;
        end;
      end;
begin
  Result := False;
  For i := 0 to Query.count-1 do
  begin
    iQuery := TQPQuery(Query[i]);
  
    Case iQuery.mode of
      qmName : Result := RomCmp(iQuery.Filter, _name, iQuery.FullMatch);
      qmFileName : Result := RomCmp(iQuery.Filter, ExtractFileName(_path), iQuery.FullMatch);
      qmEmu : Result := RomCmp(iQuery.Filter, _emulator, iQuery.FullMatch);
      qmCompany : Result := RomCmp(iQuery.Filter, _company, iQuery.FullMatch);
      qmYear : Result := RomCmp(iQuery.Filter, _year, iQuery.FullMatch);
      qmType : Result := RomCmp(iQuery.Filter, _GameType, iQuery.FullMatch);
      qmNumPlay : Result := RomCmp(iQuery.Filter, IntToStr(_NumPlay), iQuery.FullMatch);
      qmPlayers : Result := RomCmp(iQuery.Filter, _MultiPlayer, iQuery.FullMatch);
      qmRating : Result := RomCmp(iQuery.Filter, _Rating, iQuery.FullMatch);
    end;

    //If we only need to match one element and its found then exit.
    if (Result) and (not MatchAll) then
      Exit;

    //we need to match all the criteria, and it hasnt (hence the false), so no
    //need to carry on in the loop.  
    if (not Result) and (MatchAll) then
      Exit;

  end;
end;

{-----------------------------------------------------------------------------}

procedure TQPRom.RenameFile(NewName : String);
var
  Find : ThhFindFile;
  i : Integer;
begin
  If NewName <> '' then
  begin

    //we need to rename all the 'extra' files for this ROM.  Save state etc..
    Find := ThhFindFile.Create(nil);
    Try

      Find.Directory := ExtractFilePath(_path);
      Find.Filter := ExtractFileName(ChangeFileExt(_Path, '')) + '.*';
      Find.Recurse := False;

      find.Execute;

      For I := 0 to Find.TotalFile-1 do
        SysUtils.RenameFile(Find.Files[i], ExtractFilePath(Find.Files[i]) + ChangeFileExt(NewName, ExtractFileExt(Find.Files[i])));

      SysUtils.RenameFile(_path, ExtractFilePath(_path)+ NewName);

      _path := ExtractFilePath(_path)+ NewName;

    Finally
      FreeAndNil(Find);
    end;

  end
  else
    raise EJException.create(QP_INVALID_FILE_NAME);
end;

{-----------------------------------------------------------------------------}

procedure TQPRom.RunIPS(ExtrDir : String; Emu : TObject; IPSIndex : Integer);
Var
  i : Integer;
  RPath, IPSpath : String;
  ZContents : TStrings;
  tmpRom : TQPRom;
  Patcher : TIPSPatcher;
begin

    IPSPath := self.IPS[IPSIndex].Path;

    //now create the temp directory again.
    If Not DirectoryExists(ExtrDir) then
      CreateDir(ExtrDir);

    //check that IPS file actually exists!
    if not FileExists(IPSPath) then
      raise EJException.Create(QP_IPS_NOT_FOUND);

    //check that the ROM file exists.
    if not FileExists(_path) then
      raise EJException.Create(QP_ROM_NOT_FOUND);

    if lowercase(extractFileExt(_Path)) = '.zip' then
    Begin

      UnzipZip(_path, ExtrDir, False, False, False);
      zContents := TStringList.Create;
      try

        GetLastZipContents(zContents);
        //if no files are found the extraction has failed
        If zContents.Count = 0 then
          raise EJException.Create(QP_ZIP_NO_FILE_AFTER_EXTRACT);

        //if one file was extracted this is the rom, save the path
        If zContents.Count = 1 then
          RPath := ExtrDir + ExtractFilename(zContents[0]);

        //if more than one file is extracted we have a problem!
        //Just try to GUESS which file is the rom... pants...
        If zContents.Count > 1 then
          For i := 0 to zContents.Count-1 do
            If (ExtractFileExt(zContents[i]) <> '.txt') or
               (ExtractFileExt(zContents[i]) <> '.nfo') or
               (ExtractFileExt(zContents[i]) <> '.diz') then
            Begin
              RPath := ExtrDir + ExtractFileName(zContents[i]);
              Break;
            End;

      finally
        FreeAndNil(zContents);
      end;

    end //end of IF fileEXT is ZIP
    else
    Begin //we dont need to extract zip file, just copy the rom to the temp dir
      FileCopy(RPath, ExtrDir + ExtractFileName(RPath));

      If FileExists(ExtrDir+ExtractFileName(RPath)) then
        RPath := ExtrDir+ExtractFileName(RPath);
    end;

    //by this stage RPath, WILL point to a file in the temp directory.

    //now its time to combine the rom and the patch
    // - RPath, IPSpath
  try
    Patcher := TIPSPatcher.Create(IPSpath, RPath);
    try
      Patcher.Execute;
    finally
      FreeAndNil(Patcher);
    end;

    //Now run the IPS game
    tmpRom := TQPRom.Create();
    Try
      tmpRom.Path := rPath;
      //This can throw exceptions...
      TQPEmu(Emu).RunRom(tmpRom);
    Finally
      FreeAndNil(tmpRom);
    end;

  finally
    //clean out the temp folder
    DeleteFile(Rpath);
  end;

end;

Procedure TQPRom.SetLanguage();
Const
  CCode: Array [0..19] of String = ('(1)', '(A)', '(C)', '(F)', '(FC)', '(FN)', '(G)', '(GR)', '(HK)', '(H)', '(J)', '(K)', '(NL)', '(S)', '(SW)', '(U)', '(US)', '(UK)', '(I)', '(UnK)');
  Lang : Array [0..19] of String = (QP_JAP, QP_ENG, QP_CHI, QP_FRE, QP_FRE, QP_FIN, QP_GER, QP_GRE, QP_CHI, QP_DUT, QP_JAP, QP_KOR, QP_DUT, QP_SPA, QP_SWE, QP_ENG, QP_ENG,QP_ENG, QP_ITA, QP_ENG);
var
  I : Integer;
Begin
  //this code attempts to guess the language of a filename
  //by scanning the filename for any known country codes
  //obviously if the file name has no country code - it wont work :-)

  _Language := QP_ENG;
  
  For I := 0 to 19 do
    If Pos(lowercase(CCode[i]), lowercase(ExtractFileName(_path))) <> 0 then
    Begin
      _Language := Lang[i];
      Break;
    end;

End;

procedure TQPRom.SetMameMultiplayer(Ini : TMemIniFile);
var
  Temp : String;  // Variable for holding search criteria.
Begin

  //This function determines a games multiplayer mode. For MAME only!!

  // if it has a shortname then use this to search with
  // if it doesnt then try to extract the zip file name and work from that.
  If _CloneName <> '' then
    Temp := _CloneName
  else
    Temp := ChangeFileExt(Extractfilename(_path), '');

  Temp := ini.ReadString('NPlayers', temp , 'Unknown');

  _Multiplayer := Temp;
end;

Procedure TQPRom.SetMameGameCategory(Ini : TMemIniFile);
var
  Temp : String;  // Variable for holding search criteria.
Begin

  //This function determines a games category. For MAME only!!

  // if it has a shortname then use this to search with
  // if it doesnt then try to extract the zip file name and work from that.
  If _CloneName <> '' then
    Temp := _CloneName
  else
    Temp := ChangeFileExt(Extractfilename(_path), '');

  Temp := ini.ReadString('Category', temp , 'Unknown');

  StrReplace(Temp, ' / ','/');

  _GameType := Temp;
End;

{-----------------------------------------------------------------------------}
Procedure TQPROM.ToCSV(var CodeList : TStrings; Opt : TQPExportOpt);
var
  Line : String;
begin

  if Opt.AddName then
    Line := _name + ',';

  if (Opt.AddEmu) then
    Line := Line + _emulator + ',';

  if (Opt.AddPath) then
    Line := Line + _path + ',';

  if (Opt.AddType) then
    Line := Line + _GameType + ',';

  if (Opt.AddLanguage) then
    Line := Line + _Language + ',';

  if (Opt.AddRating) then
    Line := Line + _Rating + ',';

  if (Opt.AddYear) then
    Line := Line + _Year + ',';

  if (Opt.AddPlayers) then
    Line := Line + _Multiplayer + ',';

  CodeList.add(Line);
end;

Procedure TQPROM.ToHTML(var CodeList : TStrings; EList : TObjectList; Opt : TQPExportOpt);
var
  emu : TQPEmu;
begin
  CodeList.add('<div class="ROM">');

  if Opt.AddName then
    CodeList.add('<div class="name">' + _name + '</div>');

  if (Opt.AddEmu) and (_Emulator <> '') then
    if Opt.AddWWW then
    begin
      emu := TQPEmuList(EList).GetEmuForROM(Self);
      If (emu <> nil) and (emu.HomePage <> '') then
        CodeList.Add('<div class="info">' + QP_ROM_EMU + ': <a href="' + emu.HomePage + '">' + _emulator + '</a></div>')
      else
        CodeList.add('<div class="info">' + QP_ROM_EMU + ': ' + _emulator + '</div>');
    end
    else
      CodeList.add('<div class="info">' + QP_ROM_EMU + ': ' + _emulator + '</div>');

  if (Opt.AddPath) and (_path <> '') then
    CodeList.add('<div class="info">' + QP_ROM_PATH + ': ' + _path + '</div>');

  if (Opt.AddType) and (_GameType <> '') then
    CodeList.add('<div class="info">' + QP_ROM_TYPE + ': ' + _GameType + '</div>');

  if (Opt.AddLanguage) and (_Language <> '') then
    CodeList.add('<div class="info">' + QP_ROM_LANG + ': ' + _Language + '</div>');

  if (Opt.AddRating) and (_Rating <> '') then
    CodeList.add('<div class="info">' + QP_ROM_RATING + ': ' + _Rating + '</div>');

  if (Opt.AddYear) and (_Year <> '') then
    CodeList.add('<div class="info">' + QP_ROM_YEAR + ': ' + _Year + '</div>');

  if (Opt.AddPlayers) and (_Multiplayer <> '') then
    CodeList.add('<div class="info">' + QP_ROM_PLAYERS + ': ' + _Multiplayer + '</div>');

  CodeList.add('</div>');
end;

{-----------------------------------------------------------------------------}

Function TQPROM.ToString() : String;
var
  ROMstring, Comment : String;
  I : Integer;
Begin
  RomString := _Name + '';
  RomString := RomString + _CloneName + '';
  RomString := RomString + _ParentName + '';
  RomString := RomString + _ZipName + '';
  RomString := RomString + _path + '';
  RomString := RomString + _emulator + '';
  RomString := RomString + _Company + '';
  RomString := RomString + _Year + '';
  RomString := RomString + _GameType + '';
  RomString := RomString + _Rating + '';
  RomString := RomString + _Language + '';
  RomString := RomString + _Parameters + '';
  //we need to convert returns to a numeric display
  Comment := _Comment;
  StrReplace(Comment, chr(10), '#10', [rfReplaceAll]);
  StrReplace(Comment, chr(13), '#13', [rfReplaceAll]);
  RomString := RomString + Comment + '';

  RomString := RomString + IntToStr(_NumPlay) + '';
  RomString := RomString + BoolToStr(_Overwrite) + '';
  RomString := RomString + '<IPS>' + '';

  For I := 0 to length(IPS)-1 do
    if (IPS[i].Name <> '') AND (IPS[i].Path <> '') then
    Begin
      RomString := RomString + IPS[i].Name + '';
      RomString := RomString + IPS[i].Path + '';
    end;

  RomString := RomString + '</IPS>';

  RomString := RomString + '' + _MultiPlayer;

  Result := ROMString;
End;

{-----------------------------------------------------------------------------}
Procedure TQPROM.ToXML(var CodeList : TStrings; EList : TObjectList; Opt : TQPExportOpt);
var
  emu : TQPEmu;
  temp : string;
begin
  CodeList.add('<rom>');

  if Opt.AddName then
  begin
    temp := _name;
    StrReplace(temp, '&', '&amp;', [rfReplaceAll]);
    CodeList.add('<name>' + temp + '</name>');
  end;

  if (Opt.AddEmu) and (_Emulator <> '') then
    if Opt.AddWWW then
    begin
      emu := TQPEmuList(EList).GetEmuForROM(self);
      If (emu <> nil) and (emu.HomePage <> '') then
        CodeList.Add('<emulator>' + _emulator + '<homepage>' + emu.HomePage + '</homepage></emulator>')
      else
        CodeList.add('<emulator>' + _emulator + '</emulator>');
    end
    else
      CodeList.add('<emulator>' + _emulator + '</emulator>');

  if (Opt.AddPath) and (_path <> '') then
    CodeList.add('<path>' + _path + '</path>');

  if (Opt.AddType) and (_GameType <> '') then
  begin
    temp := _GameType;
    StrReplace(temp, '&', '&amp;', [rfReplaceAll]);
    CodeList.add('<type>' + temp + '</type>');
  end;

  if (Opt.AddLanguage) and (_Language <> '') then
    CodeList.add('<language>' + _Language + '</language>');

  if (Opt.AddRating) and (_Rating <> '') then
    CodeList.add('<rating>' + _Rating + '</rating>');

  if (Opt.AddYear) and (_Year <> '') then
    CodeList.add('<year>' + _Year + '</year>');

  if (Opt.AddPlayers) and (_Multiplayer <> '') then
    CodeList.add('<players>' + _Multiplayer + '</players>');

  CodeList.add('</rom>');
end;

end.
